#!/usr/bin/python

# Example Function Call:
# python3 darm_loop_critique.py --run=O3 --IFO=H1 --modelPath=. --DARMmodelfile=modelparams_H1_20200103 --modelFunction=modelPars --outputfile=2020-01-14_critique

import argparse
#import matplotlib as mpl
#mpl.use('Agg')
from matplotlib import pyplot as plt
from scipy import signal
from datetime import date
import numpy as np


parser = argparse.ArgumentParser(description='Script that determines BNS range with a correction from a PCAL to DARM sweep')
parser.add_argument('--run', required=True, type=str, help = 'Name of the run, e.g., O3')
parser.add_argument('--IFO', required=True, type=str, choices=['H1', 'L1'], help = 'Name of the IFO')
parser.add_argument('--modelPath', type=str, required=True, help='Path to the directory containing model parameter file')
parser.add_argument('--DARMmodelfile', required=True, type=str, help = 'Model parameter file')
parser.add_argument('--modelFunction', required=True, type=str, help = 'Function name in the model parameter file')
parser.add_argument('--outputfile', required=True, type=str, help = 'output file name')
args = parser.parse_args()

#Get SVN ROOT
import os
path = os.path.abspath(__file__)
dirs = path.split('trunk')
#SVNROOT = dirs[0]
SVNROOT = '/ligo/svncommon/CalSVN/aligocalibration/' #For use in my home dir

plotDate = date.isoformat(date.today())

#Get pyDARM
import sys
sys.path.insert(0, SVNROOT+'trunk/Common/pyDARM')
sys.path.insert(1, SVNROOT+'trunk/Runs/{}/{}/params'.format(args.run, args.IFO))
import importlib
sys.path.insert(-1, args.modelPath)
mod = importlib.import_module(args.DARMmodelfile)
func = getattr(mod, args.modelFunction)
pars = func()

from src.computeDARM import computeDARM
from src.filtermath import dfreqrespZPK
from src.filtermath import freqrespZPK
from src.actuation import computeSUSfilter


freq = np.logspace(-1, 4, 10001)

[D, C, A, G, CLG, R, sensProd, actProd] = computeDARM(pars, freq)

#compute DARM_CTRL to output filter bank
UIM_umodel = actProd.umodelUIM_freqresp
PUM_umodel = actProd.umodelPUM_freqresp
TST_umodel = actProd.umodelTST_freqresp

#compute output filter bank to test mess displacement
commonTF = dfreqrespZPK(actProd.SUS2IOPdelay, 2.0*np.pi*freq/2**14)[1] * \
    actProd.totalActuationDigital_freqresp * \
    actProd.AIanalog_freqresp * \
    actProd.unknownActuationDelay
UIM_strength = commonTF * actProd.uimResponse_freqresp
PUM_strength = commonTF * actProd.pumResponse_freqresp
TST_strength = commonTF * actProd.tstResponse_freqresp

#compute optical plant response
optical_freqresp = pars.Cpars.IFOgain * signal.freqresp(sensProd.coupledCavity, 2.0*np.pi*freq)[1]

#compute actuator LOCK IN to displacement
UIMdigitalFilters = dfreqrespZPK(computeSUSfilter(pars.Apars.UIMfilters[3::]), 2.0*np.pi*freq/2**14)[1]
PUMdigitalFilters = dfreqrespZPK(computeSUSfilter(pars.Apars.PUMfilters[2::]), 2.0*np.pi*freq/2**14)[1]
TSTdigitalFilters = dfreqrespZPK(computeSUSfilter(pars.Apars.TSTfilters[1::]), 2.0*np.pi*freq/2**14)[1]
UIM_authority = UIMdigitalFilters * UIM_strength
PUM_authority = PUMdigitalFilters * PUM_strength
TST_authority = TSTdigitalFilters * TST_strength

#compute DARM IN to displacement scaled by optical gain
UIM_total = pars.Cpars.IFOgain * D * UIM_authority
PUM_total = pars.Cpars.IFOgain * D * PUM_authority
TST_total = pars.Cpars.IFOgain * D * TST_authority

#Products needed for actuator measurements
PUM_LOCKfilter = dfreqrespZPK(computeSUSfilter([pars.Apars.PUMfilters[2]]), 2.0*np.pi*freq/2**14)[1]
PUM_DRIVEALIGN_2_disp = PUM_authority / PUM_LOCKfilter
TST_ISCINFfilter = dfreqrespZPK(computeSUSfilter([pars.Apars.TSTfilters[0]]), 2.0*np.pi*freq/2**14)[1]
TST_LOCKfilter = dfreqrespZPK(computeSUSfilter([pars.Apars.TSTfilters[1]]), 2.0*np.pi*freq/2**14)[1]
TST_DRIVEALIGN_2_disp = TST_authority / TST_LOCKfilter
UIM_meas = - UIM_authority / (1.0 + C*D*TST_ISCINFfilter*TST_authority + C*D*TST_ISCINFfilter*TST_LOCKfilter*PUM_authority)
PUM_meas = - C * D * TST_ISCINFfilter * TST_LOCKfilter * PUM_LOCKfilter * (PUM_DRIVEALIGN_2_disp + UIM_authority) / (1.0 + C*D*TST_ISCINFfilter*TST_authority)
TST_meas = - C * D * TST_ISCINFfilter * TST_LOCKfilter * PUM_LOCKfilter * (PUM_DRIVEALIGN_2_disp * UIM_authority * TST_DRIVEALIGN_2_disp / PUM_LOCKfilter)

# Sheila requested some additional data products be produced based on the following diagram
#                   EXC_U                
#                IN1_U|IN2_U 
#                  ^  V  ^        
#               +——+—|+|—+—>|  U  |————>—+
#               |   EXC_P                |
#               |IN1_P|IN2_P             |
#               |  ^  V  ^               V
#               +——+—|+|—+—>|  P  |————>|+|
#               |   EXC_E                |
#               |IN1_E|IN2_E             |
#               |  ^  V  ^               V
# +——| C |—————>+——+—|+|—+—>|  E  |————>|+|
# ^                                      |
# |                                      |
# +——————————————<———————————————————————+
# 
# (1) C*(E + P + U) compared against the open loop gain, G, and in that plot, showing C*E, C*P, and C*U.
# (2) E / (P+U), P/(E+U), and U/(E+P) — this is to check that in each ratio, that where the magnitude of the function crosses zero, the phase is *not* 180 deg (i.e. the stages aren’t canceling each other and thus causing an instability).
# (3) IN1_P / IN2_P = C(P*U) / (1 - C*E) === PUM_meas
# (4) IN1_U / IN2_U = C*U*(L2 LOCK) / (1 - C[E+P*L2LOCK]) === UIM_meas
# where (3) and (4) are what we’re calling the “crossover” measurements.
test1 = C*D*TST_ISCINFfilter*TST_LOCKfilter*(TST_authority/TST_LOCKfilter + PUM_authority + UIM_authority)
test1_L3 = C*D*TST_ISCINFfilter*TST_authority
test1_L2 = C*D*TST_ISCINFfilter*TST_LOCKfilter*PUM_authority
test1_L1 = C*D*TST_ISCINFfilter*TST_LOCKfilter*UIM_authority
test2_L3 = TST_authority / (PUM_authority + UIM_authority)
test2_L2 = PUM_authority / (TST_authority + UIM_authority)
test2_L1 = UIM_authority / (TST_authority + PUM_authority)
# test3 and test4 are done by PUM_meas and UIM_meas above

#resultsDir = SVNROOT+'trunk/Runs/{}/{}/Results/Critique/'.format(args.run, args.IFO)
resultsDir = '/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/3/Results/' #For use in my home dir

figTag = plotDate+'_'+args.run+'_'+args.IFO+'_DARMLoopCritique'

#plots
plt.figure()
plt.subplot(211)
plt.plot(freq,abs(G))
plt.xscale('log')
plt.yscale('log')
plt.ylabel('Magnitude')
plt.grid(True, which='both')
plt.xlim(0.1, 1e4)
plt.ylim(1e-8, 1e10)
plt.subplot(212)
plt.plot(freq,np.angle(G, deg=True))
plt.xscale('log')
plt.ylabel('Phase (deg.)')
plt.xlabel('Frequency (Hz)')
plt.xlim(0.1, 1e4)
plt.ylim(-185, 185)
plt.grid(True, which='both')
plt.suptitle('{} open loop gain'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_OLGTF.pdf', bbox_inches='tight')

plt.figure()
plt.subplot(211)
plt.plot(freq,abs(G))
plt.xscale('log')
plt.yscale('log')
plt.ylabel('Magnitude')
plt.grid(True, which='both')
plt.xlim(10, 200)
plt.ylim(1e-1, 1e2)
plt.subplot(212)
plt.plot(freq,np.angle(G, deg=True))
plt.xscale('log')
plt.ylabel('Phase (deg.)')
plt.xlabel('Frequency (Hz)')
plt.xlim(10, 200)
plt.ylim(-185, 185)
plt.grid(True, which='both')
plt.suptitle('{} open loop gain zoom'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_OLGTF_zoom.pdf', bbox_inches='tight')

plt.figure()
plt.plot(freq,abs(1./(1.+G)))
plt.xscale('log')
plt.yscale('log')
plt.xlabel('Frequency (Hz)')
plt.ylabel('Magnitude')
plt.xlim(0.1, 1e4)
plt.ylim(1e-10, 1e2)
plt.grid(True, which='both')
plt.suptitle('{} closed loop gain'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_CLGTF.pdf', bbox_inches='tight')

plt.figure()
plt.subplot(211)
plt.plot(freq, abs(D))
plt.xscale('log')
plt.yscale('log')
plt.ylabel('Magnitude')
plt.xlim(0.1, 1e3)
plt.ylim(1, 1e15)
plt.grid(True, which='both')
plt.subplot(212)
plt.plot(freq,np.angle(D, deg=True))
plt.xscale('log')
plt.ylabel('Phase (deg.)')
plt.xlabel('Frequency (Hz)')
plt.xlim(0.1, 1e3)
plt.ylim(-185, 185)
plt.grid(True, which='both')
plt.suptitle('{} DARM filter'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_DARMfilter.pdf', bbox_inches='tight')

plt.figure()
plt.subplot(211)
plt.plot(freq, abs(C))
plt.xscale('log')
plt.yscale('log')
plt.ylabel('Magnitude')
plt.xlim(1, 1e4)
plt.ylim(1, 1e15)
plt.grid(True, which='both')
plt.subplot(212)
plt.plot(freq,np.angle(C, deg=True))
plt.xscale('log')
plt.ylabel('Phase (deg.)')
plt.xlabel('Frequency (Hz)')
plt.ylim(-185, 185)
plt.xlim(1, 1e4)
plt.grid(True, which='both')
plt.suptitle('{} optical plant response'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_OpticalResponse.pdf', bbox_inches='tight')

plt.figure()
plt.subplot(211)
plt.plot(freq, abs(UIM_strength), label='UIM')
plt.plot(freq, abs(PUM_strength), label='PUM')
plt.plot(freq, abs(TST_strength), label='TST')
plt.xscale('log')
plt.yscale('log')
plt.ylabel('Magnitude')
plt.xlim(0.1, 1e4)
plt.ylim(1e-26, 1e-8)
plt.grid(True, which='both')
plt.legend()
plt.subplot(212)
plt.plot(freq, np.angle(UIM_strength, deg=True), label='UIM')
plt.plot(freq, np.angle(PUM_strength, deg=True), label='PUM')
plt.plot(freq, np.angle(TST_strength, deg=True), label='TST')
plt.xscale('log')
plt.ylabel('Phase (deg.)')
plt.xlabel('Frequency (Hz)')
plt.xlim(0.1, 1e4)
plt.ylim(-185, 185)
plt.grid(True, which='both')
plt.suptitle('{} output filter bank to test mass displacement'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_ActuatorStrength.pdf', bbox_inches='tight')

plt.figure()
plt.subplot(211)
plt.plot(freq, abs(UIM_umodel), label='UIM')
plt.plot(freq, abs(PUM_umodel), label='PUM')
plt.plot(freq, abs(TST_umodel), label='TST')
plt.xscale('log')
plt.yscale('log')
plt.ylabel('Magnitude')
plt.xlim(0.1, 1e3)
plt.ylim(1e-3, 1e6)
plt.grid(True, which='both')
plt.legend()
plt.subplot(212)
plt.plot(freq, np.angle(UIM_umodel, deg=True), label='UIM')
plt.plot(freq, np.angle(PUM_umodel, deg=True), label='PUM')
plt.plot(freq, np.angle(TST_umodel, deg=True), label='TST')
plt.xscale('log')
plt.ylabel('Phase (deg.)')
plt.xlabel('Frequency (Hz)')
plt.xlim(0.1, 1e3)
plt.ylim(-185, 185)
plt.grid(True, which='both')
plt.suptitle('{} DARM CTRL to output filter bank'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_SUSfilters.pdf', bbox_inches='tight')

plt.figure()
plt.plot(freq, abs(UIM_authority), label='UIM')
plt.plot(freq, abs(PUM_authority), label='PUM')
plt.plot(freq, abs(TST_authority), label='TST')
plt.plot(freq, abs(UIM_authority + PUM_authority + TST_authority), label='Total')
plt.xscale('log')
plt.yscale('log')
plt.xlabel('Frequency (Hz)')
plt.ylabel('Magnitude')
plt.xlim(0.1, 1e4)
plt.ylim(1e-21, 1e-6)
plt.grid(True, which='both')
plt.legend()
plt.suptitle('{} LOCK IN to displacement'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_ActuatorAuthority.pdf', bbox_inches='tight')

plt.figure()
plt.subplot(211)
plt.plot(freq, abs(UIM_authority), label='UIM')
plt.plot(freq, abs(PUM_authority), label='PUM')
plt.plot(freq, abs(TST_authority), label='TST')
plt.plot(freq, abs(UIM_authority + PUM_authority + TST_authority), label='Total')
plt.xscale('log')
plt.yscale('log')
plt.xlabel('Frequency (Hz)')
plt.ylabel('Magnitude')
plt.xlim(0.1, 1e2)
plt.ylim(1e-18, 1e-10)
plt.yticks(np.logspace(-18,-10,9))
plt.grid(True, which='both')
plt.legend()
plt.subplot(212)
plt.plot(freq, 180.0/np.pi*np.angle(UIM_authority), label='UIM')
plt.plot(freq, 180.0/np.pi*np.angle(PUM_authority), label='PUM')
plt.plot(freq, 180.0/np.pi*np.angle(TST_authority), label='TST')
plt.plot(freq, 180.0/np.pi*np.angle(UIM_authority + PUM_authority + TST_authority), label='Total')
plt.xscale('log')
plt.xlabel('Frequency (Hz)')
plt.ylabel('Phase (deg)')
plt.xlim(0.1, 1e2)
plt.ylim(-185,185)
plt.yticks(np.arange(-180,180+45,45))
plt.grid(True, which='both')
plt.suptitle('{} LOCK IN to displacement'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_ActuatorAuthority_XoverZoom.pdf', bbox_inches='tight')

plt.figure()
plt.plot(freq, abs(UIM_authority), label='UIM')
plt.plot(freq, abs(PUM_authority), label='PUM')
plt.plot(freq, abs(TST_authority), label='TST')
plt.plot(freq, abs(UIM_authority + PUM_authority + TST_authority), label='Total')
plt.xscale('log')
plt.yscale('log')
plt.xlabel('Frequency (Hz)')
plt.ylabel('Magnitude')
plt.xlim(100, 1000)
plt.ylim(1e-21, 1e-16)
plt.grid(True, which='both')
plt.legend()
plt.suptitle('{} LOCK IN to displacement'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_ActuatorAuthority_HighFZoom.pdf', bbox_inches='tight')

plt.figure()
plt.plot(freq, abs(UIM_total), label='UIM')
plt.plot(freq, abs(PUM_total), label='PUM')
plt.plot(freq, abs(TST_total), label='TST')
plt.plot(freq, abs(UIM_total + PUM_total + TST_total), label='Total')
plt.xscale('log')
plt.yscale('log')
plt.xlabel('Frequency (Hz)')
plt.ylabel('Magnitude')
plt.xlim(0.1, 1e4)
plt.ylim(1e-4, 1e10)
plt.grid(True, which='both')
plt.legend()
plt.suptitle('{} DARM IN to displacement'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_IFOCTRL.pdf', bbox_inches='tight')

plt.figure()
plt.subplot(211)
plt.plot(freq, abs(UIM_meas), label='UIM')
plt.plot(freq, abs(PUM_meas), label='PUM')
plt.plot(freq, abs(TST_meas), label='TST')
plt.xscale('log')
plt.yscale('log')
plt.ylabel('Magnitude (LOCK IN1/LOCK IN2)')
plt.xlim(0.1, 1e3)
plt.grid(True, which='both')
plt.legend()
plt.subplot(212)
plt.plot(freq, np.angle(UIM_meas, deg=True), label='UIM')
plt.plot(freq, np.angle(PUM_meas, deg=True), label='PUM')
plt.plot(freq, np.angle(TST_meas, deg=True), label='TST')
plt.xscale('log')
plt.ylabel('Phase (deg.)')
plt.xlabel('Frequency (Hz)')
plt.xlim(0.1, 1e3)
plt.ylim(-185, 185)
plt.grid(True, which='both')
plt.suptitle('{} LOCK transfer functions'.format(args.IFO))
plt.savefig(resultsDir+figTag+'_LOCK_tf.pdf', bbox_inches='tight')

plt.figure()
plt.plot(freq,abs(D*actProd.UIM/R))
plt.plot(freq,abs(D*actProd.PUM/R))
plt.plot(freq,abs(D*actProd.TST/R))
plt.plot(freq,abs((1.0/C)/R))
plt.plot(freq,np.ones(len(freq))*0.1,linestyle='dashed',linewidth=3)
plt.xscale('log')
plt.yscale('log')
plt.xlabel('Frequency (Hz)')
plt.ylabel('Relative Magnitude Contribution (arb.)')
plt.xlim(5, 2000)
plt.ylim(1e-4, 1e1)
plt.grid(ls='--',which='both')
plt.legend(('A (UIM)','A (PUM)','A (TST)','1 / C','10% Contribution'),loc='lower left')
#plt.title('{} {} DARM Loop Component Relative Contributions to Response \n{}'.format(args.IFO,args.run,args.DARMmodelfile))
plt.savefig(resultsDir+figTag+'_ContributionsToR_mag.pdf', bbox_inches='tight')
#FIXME -- figure out how to best demostrate the phase contribution to the total
#plt.subplot(212)
#plt.plot(freq,np.angle(D*actProd.UIM/R, deg=True),freq,np.angle(D*actProd.PUM/R, deg=True),freq,np.angle(D*actProd.TST/R, deg=True),freq,np.angle((1.0/C)/R, deg=True))
#plt.xscale('log')
#plt.xlabel('Frequency (Hz)')
#plt.ylabel('Phase (deg)')
#plt.xlim(5, 7000)
#plt.legend(('UIM','PUM','TST','Inv. Sensing'),loc='lower right')


import matplotlib.backends.backend_pdf
pdf = matplotlib.backends.backend_pdf.PdfPages(resultsDir+'Critique/{}.pdf'.format(args.outputfile))
for fig in range(1, plt.gcf().number + 1):
    pdf.savefig( fig )
pdf.close()
